package Catmandu::Fix::remove;

use Catmandu::Sane;

our $VERSION = '1.2011';

use Moo;
use namespace::clean;

extends 'Catmandu::Fix::remove_field';

1;

__END__

=pod

=head1 NAME

Catmandu::Fix::remove - shortcut for Catmandu::Fix::remove_field

=head1 SEE ALSO

L<Catmandu::Fix::remove_field>

=cut

