package Catmandu::Fix::cp;

use Catmandu::Sane;

our $VERSION = '1.2011';

use Moo;
use namespace::clean;

extends 'Catmandu::Fix::copy_field';

1;

__END__

=pod

=head1 NAME

Catmandu::Fix::cp - shortcut for Catmandu::Fix::copy_field

=head1 SEE ALSO

L<Catmandu::Fix::copy_field>

=cut
